/*
 *
 * Two flip/flop channels depending on pulse duration.
 *
 * Author Jean-Pierre PARISY <jpparisy@free.fr>
 *
 * Copyright (C) 2012
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#include <htc.h>
#include <pic.h>
#include <pic12f509.h>

// set fuse bits
 __CONFIG(MCLRE_OFF & CP_OFF & WDT_OFF & OSC_IntRC);

unsigned char cnt;
bit sGP0 = 0, sGP1 = 0, neutral = 0;

/* ************************************************************************* */
main (void)
{
	OPTION = ~T0CS & ~PSA & (nGPWU | nGPPU | T0SE | 0x2);
//	T0CS  = internal cycle clock (4 Mhz), this allows use of GP4 as port
//	PSA   = prescaler assigned to timer 
//	nGPWU = wake up disabled
//	nGPPU = pull up disabled
//	T0SE  = high to low source edge
//	prescaler values:	+-----+-------+
//						| 0x0 | 1:2   |
//						| 0x1 | 1:4   |
//						| 0x2 | 1:8   | <---
//						| 0x3 | 1:16  |
//						| 0x4 | 1:32  |
//						| 0x5 | 1:64  |
//						| 0x6 | 1:128 |
//						| 0x7 | 1:256 |
//						+-----+-------+	
//	Timer = 256 usec without prescaler
//	256 us * 8 = 2048 usec
//	Signal = 1.5 ms +/- 0.5 ms  => 1000 usec :: 2000 usec

	TRIS = 0b001000;						// GP3 as input, all other as output
	GPIO = 0;								// all ports off
	for (cnt = 0; cnt < 5; cnt++) {			// wait for five pulses
		do {} while (!GP3);					// wait start of pulse
		do {} while (GP3);					// wait end of pulse
	}
/* ************************************************************************* */
	for (;;) {								// loop for ever
		do {} while (!GP3);					// wait start of pulse
		TMR0 = 255 - 100;					// wait at least 100 * 8 = 800 usec
		do {} while (TMR0 >= 2);			// time elapsed, TMR0 has reached 0 
		do {} while (GP3);					// wait end of pulse
		cnt = TMR0;							// TMR0 is now between 18 and 156, neutral is 88
/* ************************************************************************* */
		if (cnt < 78) {						// 1400 usec
			if (!neutral) {					// previous state was neutral
				sGP0 = ~sGP0;				// invert sGP0
				neutral = 1;				// current state is not neutral
			}
		}
		else if (cnt > 98) {				// 1600 usec
			if (!neutral) {					// previous state was neutral
				sGP1 = ~sGP1;				// invert sGP1
				neutral = 1;				// current state is not neutral
			}
		}
		else {								// 1500 usec, neutral
			neutral = 0;					// set state to neutral
		}
		GP0 = sGP0;							// set port 0 according to toggle
		GP1 = sGP1;							// set port 1 according to toggle
	}
}
/* ************************************************************************* */
